@@ -2,8 +2,10 @@
 import os
 import uuid
 import time
+import socket
 import logging
 import decimal
+import hashlib
 import sqlite3
 import contextlib
 import threading
@@ -23,6 +25,7 @@ from starlette.middleware.sessions import SessionMiddleware
 import sqlalchemy as sa
 
 from .types import web, orm
+from . import file_utils
 
 LOGGER = logging.getLogger(__name__)
 FRONTEND_DIRECTORY = pkg_resources.resource_filename("qsl", "frontend")
@@ -35,6 +38,10 @@ class OAuthConfig(typing.NamedTuple):
     username_key: str
 
 
+class AppConfig(typing.NamedTuple):
+    single_project: typing.Optional[int]
+
+
 tls = threading.local()
 
 # pylint: disable=unused-argument
@@ -79,7 +86,9 @@ def build_session(engine):
         session.close()  # pylint: disable=no-member
 
 
-def add_clients(engine: sa.engine.Engine, oauth=None, frontend_port=None):
+def add_context(
+    engine: sa.engine.Engine, oauth=None, frontend_port=None, single_project: int = None
+):
     """Add database, S3, Oauth, and frontend port configuration to requests."""
 
     def middleware(request: fastapi.Request, call_next):
@@ -87,6 +96,7 @@ def add_clients(engine: sa.engine.Engine, oauth=None, frontend_port=None):
         request.state.s3 = boto3.client("s3")
         request.state.oauth = oauth
         request.state.frontend_port = frontend_port
+        request.state.single_project = single_project
         response = call_next(request)
         return response
 
@@ -99,6 +109,11 @@ def get_session(request: Request):
         yield session
 
 
+def get_single_project(request: Request) -> typing.Optional[int]:
+    """Get the single project if we are in single project mode."""
+    return request.state.single_project
+
+
 def get_s3(request: Request):
     """Provide an s3 client"""
     return request.state.s3
@@ -362,10 +377,18 @@ def list_projects(
     ]
 
 
+def compute_hash(filepath: str) -> str:
+    """Compute an MD5 hash for a filepath string."""
+    h = hashlib.md5()
+    h.update(filepath.encode())
+    return h.hexdigest()
+
+
 def create_images(
     group: web.ImageGroup,
     project_id: int,
     session: sa.orm.Session = fastapi.Depends(get_session),
+    s3=fastapi.Depends(get_s3),
     user: web.User = fastapi.Depends(get_current_user),
 ) -> typing.List[web.Image]:
     """Add images to a project."""
@@ -376,11 +399,12 @@ def create_images(
     ), "Did not find labeling configuratin for project."
     images = [
         orm.Image(
+            id=compute_hash(filepath),
             filepath=filepath,
             project_id=project_id,
             last_access=typing.cast(decimal.Decimal, now),
         )
-        for filepath in group.files
+        for filepath in file_utils.filepaths_from_patterns(group.files, s3=s3)
     ]
     for image in images:
         session.add(image)
@@ -422,7 +446,7 @@ def list_images(
     limit: int = None,
     max_labels: int = None,
     project_id: int = None,
-    exclude: typing.Optional[typing.List[int]] = fastapi.Query(None),
+    exclude: typing.Optional[typing.List[str]] = fastapi.Query(None),
     session: sa.orm.Session = fastapi.Depends(get_session),
     shuffle: bool = False,
     exclude_ignored: bool = False,
@@ -499,7 +523,7 @@ def list_images(
 
 def get_file(
     project_id: int,
-    image_id: int,
+    image_id: str,
     session: sa.orm.Session = fastapi.Depends(get_session),
     s3=fastapi.Depends(get_s3),
     user: web.User = fastapi.Depends(get_current_user),
@@ -521,15 +545,13 @@ def get_file(
             ExpiresIn=3600,
         )
         return fastapi.responses.RedirectResponse(
-            url, headers={"Cache-Control": "max-age=30"}
+            url, headers={"Cache-Control": "max-age=60"}
         )
     if filepath.startswith("http://") or filepath.startswith("https://"):
         return fastapi.responses.RedirectResponse(
-            filepath, headers={"Cache-Control": "max-age=30"}
+            filepath, headers={"Cache-Control": "max-age=60"}
         )
-    return fastapi.responses.FileResponse(
-        filepath, headers={"Cache-Control": "max-age=0"}
-    )
+    return fastapi.responses.FileResponse(filepath)
 
 
 def update_label_group_with_label(
@@ -553,7 +575,7 @@ def update_label_group_with_label(
 
 def get_labels(
     project_id: int,
-    image_id: int,
+    image_id: str,
     user: web.User = fastapi.Depends(get_current_user),
     session: sa.orm.Session = fastapi.Depends(get_session),
 ) -> web.ImageLabels:
@@ -628,7 +650,7 @@ def get_labels(
 
 def delete_labels(
     project_id: int,
-    image_id: int,
+    image_id: str,
     user: web.User = fastapi.Depends(get_current_user),
     session: sa.orm.Session = fastapi.Depends(get_session),
 ) -> web.ImageLabels:
@@ -646,7 +668,7 @@ def delete_labels(
 
 def set_labels(
     project_id: int,
-    image_id: int,
+    image_id: str,
     labels: web.ImageLabels,
     user: web.User = fastapi.Depends(get_current_user),
     session: sa.orm.Session = fastapi.Depends(get_session),
@@ -705,7 +727,7 @@ def set_labels(
 def query_labels(
     session: sa.orm.Session,
     project_id: int = None,
-    image_id: int = None,
+    image_id: str = None,
     user_id: int = None,
 ):
     """Build a label query."""
@@ -763,6 +785,7 @@ def query_labels(
         .join(
             orm.LabelConfig,
             (collections.c.project_id == orm.LabelConfig.project_id),
+            isouter=True,
         )
         .join(
             orm.ImageLevelLabel,
@@ -865,7 +888,14 @@ def export_project(
                 labels, box_lookup = default_label_mapping.get(
                     image_id, (web.LabelGroup(single={}, multiple={}, text={}), {})
                 )
-            if config.level == orm.Level.image:
+            if config is None:
+                # There's nothing to update if there's no
+                # label configuration. This happens if
+                # we have files in the project but the
+                # label configuration for the project is
+                # empty.
+                pass
+            elif config.level == orm.Level.image:
                 update_label_group_with_label(
                     label_group=labels,
                     config=config,
@@ -981,9 +1011,13 @@ async def login(request: Request, oauth=fastapi.Depends(get_oauth)):
 def auth_config(
     user: web.User = fastapi.Depends(get_current_user),
     oauth: OAuthConfig = fastapi.Depends(get_oauth),
+    single_project: int = fastapi.Depends(get_single_project),
 ) -> web.AuthConfig:
     """Get the authentication configuration for the application."""
-    return web.AuthConfig(provider=oauth.name if oauth is not None else None)
+    return web.AuthConfig(
+        provider=oauth.name if oauth is not None else None,
+        singleProject=single_project,
+    )
 
 
 async def auth(
@@ -1011,7 +1045,7 @@ async def auth(
     }
     if frontend_port is not None:
         return fastapi.responses.RedirectResponse(
-            url=f"http://localhost:{frontend_port}/"
+            url=f"http://{request.headers['Host'].split(':')[0]}:{frontend_port}/"
         )
     return fastapi.responses.RedirectResponse(url="/")
 
@@ -1026,7 +1060,9 @@ def frontend(path: str):
     )
 
 
-def build_oauth(provider_name: str, **kwargs) -> typing.Optional[OAuthConfig]:
+def build_oauth(
+    provider_name: str, single_project: bool = False, **kwargs
+) -> typing.Optional[OAuthConfig]:
     """Build an oauth configuration."""
     if provider_name is None:
         return None
@@ -1065,37 +1101,57 @@ def build_oauth(provider_name: str, **kwargs) -> typing.Optional[OAuthConfig]:
     raise NotImplementedError(f"Unsupported Oauth provider: {provider_name}.")
 
 
-def default_startup(app: fastapi.FastAPI):
+def mock_user_middleware(user: web.User):
+    """Middleware to automatically insert the user for the simple app."""
+
+    def middleware(request: fastapi.Request, call_next):
+        request.session["user"] = user.dict()
+        return call_next(request)
+
+    return middleware
+
+
+def default_startup(app: fastapi.FastAPI, engine=None):
     """Default startup for a full implementation of QSL."""
 
     def startup():
+        nonlocal engine
         CONFIG = Config(".env")
         frontend_port = None
         connection_string = CONFIG(
             "DB_CONNECTION_STRING", str, "sqlite:///qsl-labeling.db"
         )
         initial_user = CONFIG("OAUTH_INITIAL_USER", str, "Default User")
-        engine = sa.create_engine(
-            connection_string,
-            echo=False,
-            connect_args={"check_same_thread": False},
-        )
+        if engine is None:
+            connect_args = {}
+            if connection_string.startswith("sqlite://"):
+                connect_args["check_same_thread"] = False
+            engine = sa.create_engine(
+                connection_string, echo=False, connect_args=connect_args
+            )
         orm.BaseModel.metadata.create_all(engine)
         oauth = build_oauth(
             provider_name=CONFIG("OAUTH_PROVIDER", str, None),
             client_id=CONFIG("OAUTH_CLIENT_ID", str, None),
-            client_secret=CONFIG("OAUTH_CLIENTS_ECRET", str, None),
+            client_secret=CONFIG("OAUTH_CLIENT_SECRET", str, None),
         )
-        if oauth is None:
-            LOGGER.warning("Starting application with no authentication.")
 
         if CONFIG("DEVELOPMENT_MODE", bool, False):
             # We're in development mode and need to let the live frontend
             # access the backend.
             frontend_port = CONFIG("FRONTEND_PORT", str, "5000")
+
+            # Get the local network IP address so we can access
+            # services from other devices. We can't use "*"
+            # because it is not supported when credentials are
+            # passed.
+            test_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+            test_socket.connect(("8.8.8.8", 80))
+            ipaddr = test_socket.getsockname()[0]
             app.add_middleware(
                 fastapi.middleware.cors.CORSMiddleware,
                 allow_origins=[
+                    f"http://{ipaddr}:{frontend_port}",
                     f"http://localhost:{frontend_port}",
                 ],
                 allow_credentials=True,
@@ -1104,19 +1160,29 @@ def default_startup(app: fastapi.FastAPI):
             )
 
         with build_session(engine) as session:
-            if not any(
-                user.name == initial_user for user in list_users(session=session)
-            ):
-                create_user(
+            user = next(
+                (
+                    user
+                    for user in list_users(session=session)
+                    if user.name == initial_user
+                ),
+                None,
+            )
+            if user is None:
+                user = create_user(
                     new_user=web.User(name=initial_user, isAdmin=True),
                     existing_user=web.User(name="Dummy User", isAdmin=True),
                     session=session,
                 )
+            if oauth is None:
+                LOGGER.warning("Starting application with no authentication.")
+                app.middleware("http")(mock_user_middleware(user))
         app.middleware("http")(
-            add_clients(
+            add_context(
                 engine=engine,
                 oauth=oauth,
                 frontend_port=frontend_port,
+                single_project=CONFIG("SINGLE_PROJECT", int, None),
             )
         )
         app.add_middleware(
@@ -1160,16 +1226,6 @@ def build_app():
     return app
 
 
-def mock_user_middleware(user: web.User):
-    """Middleware to automatically insert the user for the simple app."""
-
-    def middleware(request: fastapi.Request, call_next):
-        request.session["user"] = user.dict()
-        return call_next(request)
-
-    return middleware
-
-
 def launch_app(host: str, port: int, log_level: str, dev: bool):
     """Launch the full QSL app."""
     uvicorn.run(
@@ -1185,16 +1241,13 @@ def launch_app(host: str, port: int, log_level: str, dev: bool):
 def launch_simple_app(host: str, port: int, project: web.Project):
     """Launch a simplified version of the QSL app."""
     user = web.User(name="Default User", id=1, isAdmin=True)
-
-    app = build_app()
     engine = sa.create_engine(
         "sqlite://",
-        echo=False,
-        connect_args={"check_same_thread": False},
         poolclass=sa.pool.StaticPool,
+        connect_args={"check_same_thread": False},
     )
-    orm.BaseModel.metadata.create_all(engine)
     with build_session(engine) as session:
+        orm.BaseModel.metadata.create_all(engine)
         project_id = create_project(project=project, session=session).id
         assert project_id is not None, "Did not get a project ID."
         user = create_user(
@@ -1229,7 +1282,7 @@ def launch_simple_app(host: str, port: int, project: web.Project):
                             )
                         filepath_to_user_labels[image.filepath] = labels.labels
             # Create images without defaults.
-            filepath_to_id: typing.Dict[str, int] = {}
+            filepath_to_id: typing.Dict[str, str] = {}
             for saved in create_images(
                 group=web.ImageGroup(files=[image.filepath for image in no_defaults]),
                 project_id=project_id,
@@ -1262,20 +1315,15 @@ def launch_simple_app(host: str, port: int, project: web.Project):
                     user=user,
                     session=session,
                 )
-    app.middleware("http")(
-        add_clients(
-            engine=engine,
-        )
+    os.environ["OAUTH_INITIAL_USER"] = user.name
+    os.environ["SINGLE_PROJECT"] = str(project_id)
+    app = build_app()
+    app.on_event("startup")(default_startup(app=app, engine=engine))
+    uvicorn.run(
+        app,
+        host=host,
+        port=port,
     )
-    app.on_event("startup")(
-        lambda: app.add_middleware(
-            SessionMiddleware,
-            secret_key="not-a-secret",
-            session_cookie="qsl-session",
-        )
-    )
-    app.middleware("http")(mock_user_middleware(user))
-    uvicorn.run(app, host=host, port=port)
     with build_session(engine) as session:
         return export_project(user=user, project_id=project_id, session=session)
 
