@@ -41,8 +41,8 @@ class Box(BaseModel):
         return hash(
             (
                 (self.x, self.y, self.w, self.h)
-                + (p.__hash__() for p in self.points)
-                + self.labels.__hash__()
+                + tuple(p.__hash__() for p in (self.points or []))
+                + (self.labels.__hash__(),)
             )
         )
 
@@ -123,7 +123,7 @@ class User(BaseModel):
 
 
 class Image(BaseModel):
-    id: typing.Optional[int]
+    id: typing.Optional[str]
     filepath: str
     labels: typing.Optional[int]
     status: typing.Optional[tx.Literal["ignored", "labeled", "unlabeled"]]
@@ -140,7 +140,7 @@ class ExportedUserLabels(BaseModel):
 
 
 class ExportedImageLabels(BaseModel):
-    imageId: typing.Optional[int]
+    imageId: typing.Optional[str]
     filepath: str
     labels: typing.List[ExportedUserLabels]
     defaultLabels: typing.Optional[ImageLabels]
@@ -159,6 +159,56 @@ class Project(BaseModel):
             return False
         return self.labelingConfiguration == other.labelingConfiguration
 
+    def get_filepaths(self) -> typing.List[str]:
+        """Get a list of the filepaths in a project."""
+        if self.labels is None:
+            raise ValueError("This project does not have labels set.")
+        return [l.filepath for l in self.labels]
+
+    def box_level_labels(self):
+        """Get the box level labels as a list of dicts."""
+        if self.labels is None:
+            raise ValueError("This project does not have labels set.")
+        rows = []
+        for image_labels in self.labels:
+            for user_label in image_labels.labels:
+                for box in user_label.labels.boxes:
+                    rows.append(
+                        dict(
+                            x=box.x,
+                            y=box.y,
+                            w=box.w,
+                            h=box.h,
+                            points=[{"x": p.x, "y": p.y} for p in box.points]
+                            if box.points
+                            else None,
+                            filepath=image_labels.filepath,
+                            **box.labels.single,
+                            **box.labels.multiple,
+                            **box.labels.text,
+                            user=user_label.userId
+                        )
+                    )
+        return rows
+
+    def image_level_labels(self):
+        """Get the image level labels as a list of dicts."""
+        if self.labels is None:
+            raise ValueError("This project does not have labels set.")
+        rows = []
+        for image_labels in self.labels:
+            for user_label in image_labels.labels:
+                rows.append(
+                    dict(
+                        filepath=image_labels.filepath,
+                        **user_label.labels.image.single,
+                        **user_label.labels.image.multiple,
+                        **user_label.labels.image.text,
+                        user=user_label.userId
+                    )
+                )
+        return rows
+
 
 class InitializationConfiguration(BaseModel):
     imageGroups: typing.List[ImageGroup]
@@ -167,3 +217,4 @@ class InitializationConfiguration(BaseModel):
 
 class AuthConfig(BaseModel):
     provider: typing.Optional[tx.Literal["github", "google"]]
+    singleProject: typing.Optional[int]
